#!/bin/bash

set -vx

export PROJECT_ID=django-app-unicodex
gcloud config set project $PROJECT_ID

gcloud config list

gcloud config set run/platform managed

export REGION=europe-west4
gcloud config set run/region $REGION

gcloud services enable \
  run.googleapis.com \
  iam.googleapis.com \
  compute.googleapis.com \
  sql-component.googleapis.com \
  sqladmin.googleapis.com \
  cloudbuild.googleapis.com \E
  cloudkms.googleapis.com \
  cloudresourcemanager.googleapis.com \
  secretmanager.googleapis.com || exit 1

export SERVICE_NAME=unicodex

gcloud iam service-accounts create $SERVICE_NAME \
  --display-name "$SERVICE_NAME service account" || exit 1

export CLOUDRUN_SA=${SERVICE_NAME}@${PROJECT_ID}.iam.gserviceaccount.com

for role in cloudsql.client run.admin; do
  gcloud projects add-iam-policy-binding $PROJECT_ID \
    --member serviceAccount:$CLOUDRUN_SA \
    --role roles/${role} || exit 1
done

export INSTANCE_NAME=${PROJECT_ID}-cloudsql
export ROOT_PASSWORD=$(cat /dev/urandom | tr -dc 'a-zA-Z0-9' | fold -w 64 | head -n 1)

gcloud sql instances create $INSTANCE_NAME \
  --database-version POSTGRES_11 \
  --tier db-f1-micro \
  --region $REGION \
  --project $PROJECT_ID \
  --root-password $ROOT_PASSWORD || exit 1

#gcloud sql instances list

export DATABASE_INSTANCE=$PROJECT_ID:$REGION:$INSTANCE_NAME

source .env
export DATABASE_NAME=unicodex
gcloud sql databases create $DATABASE_NAME \
  --instance=$INSTANCE_NAME || exit 1

export DBUSERNAME=postgres
#export DBPASSWORD=$(cat /dev/urandom | tr -dc 'a-zA-Z0-9' | fold -w 40 | head -n 1)

gcloud sql users create $DBUSERNAME \
  --password $DBPASSWORD \
  --instance $INSTANCE_NAME || exit 1

export DATABASE_URL="postgres://$DBUSERNAME:${DBPASSWORD}@//cloudsql/$PROJECT_ID:$REGION:$INSTANCE_NAME/$DATABASE_NAME"

sed -e 's:^\(DATABASE_URL.\+@db.\+\)$:#\1:' \
    -e '/^DATABASE_URL.\+cloudsql.\+$/d' \
    -e 's:^#\(GS_MEDIA_BUCKET_NAME.\+\)$:\1:' \
    -e 's:^#\(GS_STATIC_BUCKET_NAME.\+\)$:\1:' \
    -i .env
echo DATABASE_URL=\"$DATABASE_URL\" >> .env
#for i in django_settings SUPERPASS SUPERUSER; do gcloud secrets delete $i; done
gcloud run services update $SERVICE_NAME \
  --update-env-vars "DATABASE_URL=${DATABASE_URL}" || exit 1
gcloud secrets create django_settings --replication-policy automatic --data-file .env || exit 1
sed -e 's:^#\(DATABASE_URL.\+@db.\+\)$:\1:' \
    -e 's:^\(DATABASE_URL.\+cloudsql.\+\)$:#\1:' \
    -e 's:^\(GS_MEDIA_BUCKET_NAME.\+\)$:#\1:' \
    -e 's:^\(GS_STATIC_BUCKET_NAME.\+\)$:#\1:' \
    -i .env

[ -n "$GS_MEDIA_BUCKET_NAME" -a -n "$GS_STATIC_BUCKET_NAME" ] || exit 1
for GS_BUCKET_NAME in $GS_MEDIA_BUCKET_NAME $GS_STATIC_BUCKET_NAME; do
    gsutil mb -l ${REGION} gs://${GS_BUCKET_NAME}
    gsutil iam ch \
      serviceAccount:${CLOUDRUN_SA}:roles/storage.objectAdmin \
      gs://${GS_BUCKET_NAME} || exit 1
done


export PROJECTNUM=$(gcloud projects describe ${PROJECT_ID} --format 'value(projectNumber)')
export CLOUDBUILD_SA=${PROJECTNUM}@cloudbuild.gserviceaccount.com


gcloud secrets add-iam-policy-binding django_settings \
  --member serviceAccount:$CLOUDRUN_SA \
  --role roles/secretmanager.secretAccessor || exit 1

gcloud secrets add-iam-policy-binding django_settings \
  --member serviceAccount:$CLOUDBUILD_SA \
  --role roles/secretmanager.secretAccessor || exit 1


#export SUPERUSER="admin"
#export SUPERPASS=$(cat /dev/urandom | tr -dc 'a-zA-Z0-9' | fold -w 30 | head -n 1)

for SECRET in SUPERUSER SUPERPASS; do
  gcloud secrets create $SECRET --replication-policy automatic || exit 1

  echo -n "${!SECRET}" | gcloud secrets versions add $SECRET --data-file=- || exit 1

  gcloud secrets add-iam-policy-binding $SECRET \
    --member serviceAccount:$CLOUDBUILD_SA \
    --role roles/secretmanager.secretAccessor || exit 1
done

gcloud builds submit --tag gcr.io/$PROJECT_ID/$SERVICE_NAME || exit 1

gcloud run deploy $SERVICE_NAME \
  --allow-unauthenticated \
  --image gcr.io/$PROJECT_ID/$SERVICE_NAME \
  --add-cloudsql-instances $PROJECT_ID:$REGION:$INSTANCE_NAME \
  --service-account $CLOUDRUN_SA || exit 1


gcloud run services list || exit 1
export SERVICE_URL=$(gcloud run services describe $SERVICE_NAME --format "value(status.url)")
#
echo $SERVICE_URL
#
gcloud run services update $SERVICE_NAME \
  --update-env-vars "CURRENT_HOST=${SERVICE_URL}" || exit 1

for role in cloudsql.client run.admin; do
  gcloud projects add-iam-policy-binding ${PROJECT_ID} \
    --member serviceAccount:${CLOUDBUILD_SA} \
    --role roles/${role} || exit 1
done

gcloud iam service-accounts add-iam-policy-binding ${CLOUDRUN_SA} \
  --member "serviceAccount:${CLOUDBUILD_SA}" \
  --role "roles/iam.serviceAccountUser" || exit 1

# migrate and deploy
gcloud builds submit \
  --config .cloudbuild/build-migrate-deploy.yaml \
  --substitutions "_REGION=${REGION},_INSTANCE_NAME=${INSTANCE_NAME},_SERVICE=${SERVICE_NAME}" || exit 1
